from . import parser

class lua_loader():
	def __init__(self, script_basepath, game):
		self.__scripts_basepath = script_basepath
		self.__game = game
		self.__parser = parser.luaParser()
		self.__parser.setVar("package", "%s/res/scripts/?.lua" % (script_basepath), ["path"])
	
	def __del__(self):
		del self.__parser
	
	def __convertDicts(self, value):
		if isinstance(value, dict):
			keys = value.keys()
			numeric_keys_only = True
			for key in keys:
				numeric_keys_only &= isinstance(key, int)
				value[key] = self.__convertDicts(value[key])
			
			if numeric_keys_only and list(sorted(keys)) == list(range(1, len(keys)+1)):
				value = [value[k] for k in sorted(keys)]
			
		return value

	def load_lua_script(self, filename, run, errors):
		index = 0
		if filename.startswith(self.__scripts_basepath+"/mods/"):
			index = len(self.__scripts_basepath+"/mods/")
		elif self.__game == "TPF":
			if "/local/staging_area/" in filename:
				index = filename.find("/local/staging_area/")+len("/local/staging_area/")
			elif "/workshop/content/446800/" in filename:
				index = filename.find("/workshop/content/446800/")+len("/workshop/content/446800/")

		if index > 0:
			modFolder = filename[:index]+filename[index:].partition("/")[0]
			self.__parser.setVar("package", "%s/res/scripts/?.lua;%s/config/?.lua;%s/res/scripts/?.lua" % (self.__scripts_basepath, modFolder, modFolder), ["path"])
		
		try:
			script = self.__parser.loadScriptFile(filename)
		except SyntaxError as error:
			errors.append({"level": 3, "message": "Error while parsing file: %s. The Lua parser stopped with the following message:\nline %i: %s." % (error.filename, error.lineno, error.args[0])})
		else:
			result = None
			try:
				self.__parser.executeScript(script)
				if run:
					result = self.__parser.getVar("data")()
			except SyntaxError as error:
				errors.append({"level": 3, "message": "Error while parsing file: %s. The Lua parser stopped with the following message:\nline %i: %s." % (error.filename, error.lineno, error.args[0])})
			except (ValueError, TypeError, RuntimeError) as error:
				errors.append({"level": 3, "message": "Error while loading file: %s. The Lua parser stopped with the following message:\n%s." % (filename, str(error))})
			else:
				if result:
					result = self.__convertDicts(result)
					return result