import bpy
from ..tf_util.base_functions import TF_base_functions
from mathutils import Vector

plugin_info = {
	"name": "Animations and Events",
	"description": "Support for animation and event import/export",
	"author": "Merk",
	"version": "0.7.0",
}

object_lists = [("", "", "")]

# event callback function
def get_events_enum_callback(self, context):
	obj = self.id_data
	action_list = []
	if "childId_string" in self:
		child_ID = get_children_id(self, obj, self["childId_string"]) - 1
	elif "childId" in self:
		child_ID = self["childId"]
	else:
		child_ID = 0
	
	if child_ID >= len(obj.children):
		child_ID = len(obj.children)-1

	if child_ID >= 0:
		obj = obj.children[child_ID]
	elif child_ID == -1:
		obj = obj

	if obj.animation_data and obj.animation_data.action:
		name = obj.animation_data.action.name
		list_entry = (name, name, name)
		if not list_entry in action_list:
			if not list_entry in object_lists:
				object_lists.append(list_entry)
			action_list.append(object_lists[object_lists.index(list_entry)])

	if obj.type == 'MESH':
		anims = obj.data.tf_anims
	else:
		anims = obj.tf_anims
	
	for i in range(len(anims)):
		name = anims[i].anim_name
		list_entry = (name, name, name)
		if not list_entry in action_list:
			if not list_entry in object_lists:
				object_lists.append(list_entry)
			action_list.append(object_lists[object_lists.index(list_entry)])

	if child_ID >= 0:
		for i in range(len(obj.tf_events)):
			name = obj.tf_events[i].event_name
			list_entry = (name, name, name)
			if not list_entry in action_list:
				if not list_entry in object_lists:
					object_lists.append(list_entry)
				action_list.append(object_lists[object_lists.index(list_entry)])

	if len(action_list) == 0:
		action_list.append(object_lists[0])

	return action_list

# Children callback functions
def get_children(self, context):
	obj = self.id_data
	list_entry = (obj.name, obj.name, obj.name)
	if not list_entry in object_lists:
		object_lists.append(list_entry)
	children = [object_lists[object_lists.index(list_entry)]]

	for child in obj.children:
		list_entry = (child.name, child.name, child.name)
		if not list_entry in object_lists:
			object_lists.append(list_entry)
		children.append(object_lists[object_lists.index(list_entry)])

	return children

def get_children_id(self, obj, name):
	if name in bpy.data.objects and bpy.data.objects[name] in obj.children:
		child_id = obj.children.index(bpy.data.objects[name])+1
	elif obj.name == name:
		child_id = 0
	elif "childId" in self:
		child_id = self["childId"]
	else:
		child_id = 0
	return child_id

# Event data classes
class TFEventEntry(bpy.types.PropertyGroup):
	def get_name(self):
		action_list = get_events_enum_callback(self, bpy.context)
		if "name" in self and (self["name"], self["name"], self["name"]) in action_list:
			index = action_list.index((self["name"], self["name"], self["name"]))
		else:
			index = 0
		return index

	def set_name(self, value):
		action_list = get_events_enum_callback(self, bpy.context)
		self["name"] = action_list[value][0]

	def get_child_id(self):
		index = get_children_id(self, self.id_data, self["childId_string"])
		if index == 0:
			self["childId_string"] = self.id_data.name
		return index

	def set_child_id(self, value):
		self["childId"] = value
		if value == 0:
			self["childId_string"] = self.id_data.name
		else:
			self["childId_string"] = self.id_data.children[value-1].name

	childId = bpy.props.EnumProperty(
		items = get_children,
		get = get_child_id,
		set = set_child_id
	)
	name_select = bpy.props.EnumProperty(
		items = get_events_enum_callback,
		get = get_name,
		set = set_name
	)
	forward = bpy.props.BoolProperty(default=True)

class TFEventEntryAddOp(bpy.types.Operator):
	bl_label = "Add event entry"
	bl_idname = "collection.tfevententry_add"

	index = bpy.props.IntProperty(min = 0)

	def invoke(self, context, event):
		obj = context.object
		if obj.type == "MESH":
			obj = obj.data
		collection = obj.tf_events[self.index].data
		item = collection.add()
		item.name = item.name_select
		obj.tf_events[self.index].data_activeidx = len(collection)-1
		return {'FINISHED'}

class TFEventEntryRemoveOp(bpy.types.Operator):
	bl_label = "Remove event entry"
	bl_idname = "collection.tfevententry_remove"

	index = bpy.props.IntProperty(min = 0)

	def invoke(self, context, event):
		obj = context.object
		if obj.type == "MESH":
			obj = obj.data
		collection = obj.tf_events[self.index].data
		idx = obj.tf_events[self.index].data_activeidx
		collection.remove(idx)
		if idx > 0:
			obj.tf_events[self.index].data_activeidx = idx - 1
		return {'FINISHED'}

class TFEventEntryItemUIList(bpy.types.UIList):
	def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
		layout.prop(item, "childId", "Child")
		layout.prop(item, "name_select", "Name")
		layout.prop(item, "forward", "Forward")

# Event settings classes
class TFEventSetting(bpy.types.PropertyGroup):
	event_name = bpy.props.StringProperty()
	data = bpy.props.CollectionProperty(type = TFEventEntry)
	data_activeidx = bpy.props.IntProperty(min = 0)

	@classmethod
	def register(cls):
		bpy.types.Object.tf_events = bpy.props.CollectionProperty(
						type = cls,
					)
		bpy.types.Object.tf_events_activeidx = bpy.props.IntProperty(min = 0)

	@classmethod
	def unregister(cls):
		del bpy.types.Object.tf_events
		del bpy.types.Object.tf_events_activeidx

class TFEventAddOp(bpy.types.Operator):
	bl_label = "Add animation property"
	bl_idname = "collection.tfevent_add"

	def invoke(self, context, event):
		obj = context.object
		if obj.type == "MESH":
			obj = obj.data
		collection = obj.tf_events
		collection.add()
		obj.tf_events_activeidx = len(collection)-1
		return {'FINISHED'}

class TFEventRemoveOp(bpy.types.Operator):
	bl_label = "Remove animation property"
	bl_idname = "collection.tfevent_remove"

	def invoke(self, context, event):
		obj = context.object
		if obj.type == "MESH":
			obj = obj.data
		collection = obj.tf_events
		index = obj.tf_events_activeidx
		collection.remove(index)
		if index > 0:
			obj.tf_events_activeidx = index - 1
		return {'FINISHED'}

class TFEventSettingsItemUIList(bpy.types.UIList):
	def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
		layout.prop(item, "event_name", "Name")
		layout.template_list(
			listtype_name = "TFEventEntryItemUIList",
			list_id = data.name + data.tf_events[index].event_name,
			dataptr = data.tf_events[index],
			propname = "data",
			active_dataptr = data.tf_events[index],
			active_propname = "data_activeidx",
			rows = 1,
			maxrows = 4
		)
		col = layout.column(align=True)
		col.operator("collection.tfevententry_add", icon="ZOOMIN", text="").index = index
		col.operator("collection.tfevententry_remove", icon="ZOOMOUT", text="").index = index

# Animation settings classes
class TFAnimSetting(bpy.types.PropertyGroup):
	anim_name = bpy.props.StringProperty()

	@classmethod
	def register(cls):
		bpy.types.Mesh.tf_anims = bpy.props.CollectionProperty(
						type = cls,
					)
		bpy.types.Mesh.tf_anims_activeidx = bpy.props.IntProperty(
						min = 0,
						default = 0,
					)
		bpy.types.Object.tf_anims = bpy.props.CollectionProperty(
						type = cls,
					)
		bpy.types.Object.tf_anims_activeidx = bpy.props.IntProperty(
						min = 0,
						default = 0,
					)

	@classmethod
	def unregister(cls):
		del bpy.types.Mesh.tf_anims
		del bpy.types.Mesh.tf_anims_activeidx
		del bpy.types.Object.tf_anims
		del bpy.types.Object.tf_anims_activeidx


class TFAnimAddOp(bpy.types.Operator):
	bl_label = "Add event property"
	bl_idname = "collection.tfanim_add"

	def invoke(self, context, event):
		obj = context.object
		if obj.type == "MESH":
			obj = obj.data
		collection = obj.tf_anims
		collection.add()
		obj.tf_anims_activeidx = len(collection) - 1
		return {'FINISHED'}

class TFAnimRemoveOp(bpy.types.Operator):
	bl_label = "Remove event property"
	bl_idname = "collection.tfanim_remove"

	def invoke(self, context, event):
		obj = context.object
		if obj.type == "MESH":
			obj = obj.data
		collection = obj.tf_anims
		index = obj.tf_anims_activeidx
		collection.remove(index)
		if index > 0:
			obj.tf_anims_activeidx = index - 1
		return {'FINISHED'}

class TFAnimSettingsItemUIList(bpy.types.UIList):
	use_filter_show = False

	def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
		layout.prop_search(item, "anim_name", bpy.data, "actions", text="Name")

# Plugin class
class TFAnimImportExport():
	## Internal functions
	# Panel drawing functions
	def __draw_anim_panel(self, layout, mydata, attr=""):
		boxAni = TF_base_functions.draw_hideable_box(layout, "Animations:", "animations")
		if boxAni:
			row = boxAni.row()
			row.template_list(
				listtype_name="TFAnimSettingsItemUIList",
				dataptr=mydata,
				propname="tf_anims",
				active_dataptr=mydata,
				active_propname="tf_anims_activeidx",
				rows = 1,
				maxrows = 3
			)
			col = row.column(align=True)
			col.operator("collection.tfanim_add", icon="ZOOMIN", text="")
			col.operator("collection.tfanim_remove", icon="ZOOMOUT", text="")

			layout.separator()

	def __draw_event_panel(self, layout, mydata):
		boxAni = TF_base_functions.draw_hideable_box(layout, "Events:", "events")
		if boxAni:
			row = boxAni.row()
			row.template_list(
				listtype_name="TFEventSettingsItemUIList",
				dataptr=mydata,
				propname="tf_events",
				active_dataptr=mydata,
				active_propname="tf_events_activeidx",
				rows = 1,
				maxrows = 2
			)
			col = row.column(align=True)
			col.operator("collection.tfevent_add", icon="ZOOMIN", text="")
			col.operator("collection.tfevent_remove", icon="ZOOMOUT", text="")

			layout.separator()

	# Event import/export
	def __export_events(self, obj, obj_data, has_data=False):
		if has_data:
			objdata = obj.data
		else:
			objdata = obj

		if "events" in obj_data:
			events = obj_data["events"]
		else:
			events = {}

		for event in objdata.tf_events:
			if not event.event_name in events:
				events[event.event_name] = {}

			for entry in event.data:
				if entry.childId in bpy.data.objects and bpy.data.objects[entry.childId] in objdata.children:
					child_id = objdata.children.index(bpy.data.objects[entry.childId])+1
				else:
					child_id = 0
				if not child_id in events[event.event_name]:
					events[event.event_name][child_id] = {"name": entry.name_select.partition(".")[0], "forward": entry.forward}

		if len(events) > 0:
			obj_data["events"] = events

	def __import_events(self, obj, obj_data, has_data=False):
		if has_data:
			objdata = obj.data
		else:
			objdata = obj

		# get event data
		if "events" in obj_data:
			events = obj_data["events"]
			if len(events) == 0:
				events = {}
		else:
			events = {}

		for event_name in events.keys():
			# check if event already exists
			event_data = None
			for event in objdata.tf_events:
				if event_name == event.event_name:
					event_data = event.data

			if not event_data:
				item = obj.tf_events.add()
				item.event_name = event_name
				event_data = item.data
				is_list = False

				# get iterator over event list
				if isinstance(events[event_name], dict):
					event_iter = events[event_name].keys()
				elif isinstance(events[event_name], list):
					event_iter = range(len(events[event_name]))
					is_list = True

				for child_id in event_iter:
					entry = event_data.add()
					child = None
					if is_list:
						i = child_id
					elif child_id > 0:
						i = child_id - 1
					else:
						i = len(obj_data["children"])
						entry["childId_string"] = objdata.name

					for j in range(len(obj.children)):
						if i < len(obj_data["children"]) and "blender_object" in obj_data["children"][i]:
							child = obj_data["children"][i]["blender_object"]

					if child:
						entry["childId_string"] = child.name

					try:
						entry.name_select = events[event_name][child_id]["name"][:63]
					except:
						try:
							entry.name_select = (events[event_name][child_id]["name"] + "." + entry["childId_string"].partition(".")[0])[:63]
						except:
							pass
					entry.forward = events[event_name][child_id]["forward"]

	# Animation import/export
	def __export_anims(self, obj, obj_data, has_data=False):
		if has_data:
			objdata = obj.data
		else:
			objdata = obj
			
		name = obj.name
		
		# if (TF_base_functions.user_prefs["debug"]):
			# print("\t%s: Seeking animations in:" % name)
			# for s in obj_data:
				# print("\t\t"+s)

		if "animations" in obj_data:
			animation = obj_data["animations"]
		else:
			animation = {}
		actions = set()

		for item in objdata.tf_anims:
			# print("\t%s: found action %s" % (name, item.anim_name))
			if item.anim_name in bpy.data.actions:
				actions.add(bpy.data.actions[item.anim_name])

		if obj.animation_data and obj.animation_data.action:
			actions.add(obj.animation_data.action)

		for action in actions:
			action_name = action.name.partition(".")[0]
			if not action_name in animation:
				# print("\t%s: adding action %s with %i elements" % (name, action_name, len(action.fcurves)))
				animation[action_name] = {"type": 'KEYFRAME', "params": {"keyframes": [], "origin": [0.0, 0.0, 0.0]}}
				keyframes_raw = {}
				for i in range(0, len(action.fcurves)):
					if action.fcurves[i].data_path == "delta_location":
						for j in range(0, len(action.fcurves[i].keyframe_points)):
							k = action.fcurves[i].keyframe_points[j].co[0] * 100
							if k not in keyframes_raw:
								keyframes_raw[k] = {"time": k, "rot": [0.0, 0.0, 0.0], "transl": [0.0, 0.0, 0.0]}
							keyframes_raw[k]["transl"][action.fcurves[i].array_index] = action.fcurves[i].keyframe_points[j].co[1]
					elif action.fcurves[i].data_path == "delta_rotation_euler":
						for j in range(0, len(action.fcurves[i].keyframe_points)):
							k = action.fcurves[i].keyframe_points[j].co[0] * 100
							if k not in keyframes_raw:
								keyframes_raw[k] = {"time": k, "rot": [0.0, 0.0, 0.0], "transl": [0.0, 0.0, 0.0]}
							keyframes_raw[k]["rot"][2-action.fcurves[i].array_index] = action.fcurves[i].keyframe_points[j].co[1]/3.141592654*180

				keyframes = []
				for i in sorted(keyframes_raw.keys()):
					if (i == 0) or (keyframes_raw[i]['time'] > 0):
						keyframes.append(keyframes_raw[i])

				animation[action_name]["params"]["keyframes"] = keyframes
			# print("\t%s: action %s has %i elements" % (name, action_name, len(animation[action_name]["params"]["keyframes"])))

		if len(animation) > 0:
			obj_data["animations"] = animation

	def __add_fcurves(self, action, action_data, data, pre_datapath="delta_", action_group=""):
		scene = bpy.context.scene
		fcurves = []
		if action_data["type"] == 'KEYFRAME':
			for i in range(3):
				fcurves.append(action.fcurves.new(pre_datapath + "location", i, action_group))
				fcurves.append(action.fcurves.new(pre_datapath + "rotation_euler", i, action_group))
			for i in range(len(action_data["params"]["keyframes"])):
				keyframe = action_data["params"]["keyframes"][i]
				frame = keyframe["time"]/100
				if frame < scene.frame_start:
					scene.frame_start = frame
				if frame > scene.frame_end:
					scene.frame_end = frame
				for j in range(0,3):
					fcurves[2*j].keyframe_points.insert(frame, float(keyframe["transl"][j]))
					fcurves[5-2*j].keyframe_points.insert(frame, float(keyframe["rot"][j])*3.141592654/180)
		
		elif action_data["type"] == 'KEYFRAME_MATRIX':
			for i in range(3):
				fcurves.append(action.fcurves.new(pre_datapath + "location", i, action_group))
				fcurves.append(action.fcurves.new(pre_datapath + "rotation_euler", i, action_group))
				fcurves.append(action.fcurves.new(pre_datapath + "scale", i, action_group))
			for i in range(len(action_data["params"]["keyframes"])):
				keyframe = action_data["params"]["keyframes"][i]
				frame = keyframe["time"]/100
				if frame < scene.frame_start:
					scene.frame_start = frame
				if frame > scene.frame_end:
					scene.frame_end = frame
				transf = TF_base_functions.create_4x4matrix(keyframe["transf"])
				loc, rot, scale = transf.decompose()
				rot = rot.to_euler('ZYX')
				for j in range(3):
					fcurves[3*j].keyframe_points.insert(frame, float(loc[j]))
					fcurves[1+3*j].keyframe_points.insert(frame, float(rot[j]))
					fcurves[2+3*j].keyframe_points.insert(frame, float(scale[j]))
		
		elif action_data["type"] == 'FILE_REF':
			anim_file = TF_base_functions.get_data(data["basepath"] + "models/animation/" + action_data["params"]["id"], "ANIMATION")
			for i in range(3):
				fcurves.append(action.fcurves.new(pre_datapath + "location", i, action_group))
				fcurves.append(action.fcurves.new(pre_datapath + "rotation_euler", i, action_group))
				fcurves.append(action.fcurves.new(pre_datapath + "scale", i, action_group))
			for i in range(len(anim_file["times"])):
				frame = anim_file["times"][i]/100
				if frame < scene.frame_start:
					scene.frame_start = frame
				if frame > scene.frame_end:
					scene.frame_end = frame
				transf = TF_base_functions.create_4x4matrix(anim_file["transfs"][i])
				loc, rot, scale = transf.decompose()
				rot = rot.to_euler('ZYX')
				#rot[1] = -rot[1]
				#rot[0] = -rot[0]
				for j in range(3):
					fcurves[3*j].keyframe_points.insert(frame, float(loc[j]))
					fcurves[1+3*j].keyframe_points.insert(frame, float(rot[j]))
					fcurves[2+3*j].keyframe_points.insert(frame, float(scale[j]))
		
		if action.name.partition(".")[0] in {"drive", "idle", "idle_red", "idle_green"}:
			for fcurve in fcurves:
				fcurve.modifiers.new('CYCLES')

	def __create_animation(self, anim_data, object_name, data):
		actions = []
		if str(type(anim_data)) == "<class 'dict'>":
			for key in anim_data.keys():
				anim_name = key + '.' + object_name
				if not anim_name in bpy.data.actions:
					action = bpy.data.actions.new(anim_name)
					self.__add_fcurves(action, anim_data[key], data)

					action.use_fake_user = True
				else:
					action = bpy.data.actions[anim_name]
				actions.append(action)
		return actions

	"""def __create_wheel_drive_anim(self, obj):
		obj_data = obj.data
		object_name = obj_data.name.partition(".")[0]
		if ("drive." + object_name) in bpy.data.actions:
			action = bpy.data.actions["drive." + object_name]
		else:
			pi = 3.141592654
			action = bpy.data.actions.new("drive." + object_name)
			#action.use_fake_user = True
			fcurve = action.fcurves.new("rotation_euler", 1)
			keyframe = fcurve.keyframe_points.insert(0, 0)
			keyframe.interpolation = 'LINEAR'
			fcurve.keyframe_points.insert(10*2*pi*obj.location.z, 2*pi)
			fcurve.modifiers.new('CYCLES')

		obj.animation_data_create()
		obj.animation_data.action = action"""

	def __import_anims(self, obj, object_data, data, has_data=False):
		if has_data:
			obj_data = obj.data
		else:
			obj_data = obj

		name = obj_data.name

		# if (TF_base_functions.user_prefs["debug"]):
			# print("\t%s: Seeking animations in:" % name)
			# for str in object_data:
				# print("\t\t"+str)

		if "animations" in object_data:
			# if (TF_base_functions.user_prefs["debug"]):
				# print("\tFound animations in ", name)
			actions = self.__create_animation(object_data["animations"], name, data)
			if len(actions) > 0:
				obj.animation_data_create()
				obj.animation_data.action = actions[0]
				obj.rotation_mode = 'ZYX'

				# get existing anims
				action_set = set()
				for item in obj_data.tf_anims:
					action_set.add(item.anim_name)

				# link new anims
				for i in range(len(actions)):
					if not actions[i].name in action_set:
						collection = obj_data.tf_anims
						item = collection.add()
						item.anim_name = actions[i].name

	"""def __create_bone_anim(self, obj, data, bone_data):
		actions = []
		name = obj.name.partition(".")[0]

		#output = open("C:/Users/Markus/Desktop/debug.txt", 'w')
		tf_mesh_mdl.write_var_struct(output, bone_data, "", 0, True, False)
		output.close()

		# actions = {"walk": [{"name": "bone", "data": {"type": "", "params": {}}}]}
		for action_name, action_data in bone_data.items():
			action = bpy.data.actions.new(action_name + '.' + name)
			for bone in action_data:
				if bone["name"] == "":
					self.add_fcurves(action, bone["data"])
				else:
					action.groups.new(bone["name"])
					self.__add_fcurves(action, bone["data"], "pose.bones[\"" + bone["name"] + "\"].", bone["name"])
			actions.append(action)

		if len(actions) > 0:
			obj.animation_data_create()
			obj.animation_data.action = actions[0]
			obj.rotation_mode = 'ZYX'

			action_set = set()
			for item in obj.tf_anims:
				action_set.add(item.anim_name)

			for i in range(len(actions)):
				if not actions[i].name in action_set:
					collection = obj.tf_anims
					item = collection.add()
					item.anim_name = actions[i].name"""

	## External functions
	# UI functions
	def draw_mesh_settings_panel(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport draw_mesh_settings_panel")
		self.__draw_anim_panel(data["layout"], data["mydata"], "data")

	def draw_group_settings_panel(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport draw_group_settings_panel")
		self.__draw_anim_panel(data["layout"], data["mydata"])
		self.__draw_event_panel(data["layout"], data["mydata"])

	def draw_lod_settings_panel(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport draw_lod_settings_panel")
		self.__draw_event_panel(data["layout"], data["mydata"])

	# import/export functions
	def validate_model_data(self, data): # Reset the timeline - has nothing to do with validation, but this is a good time for it
		scene = bpy.context.scene
		scene.frame_current = 0
		scene.frame_start = 0
		scene.frame_end = 0
	
	def save_model_data_2(self, data):
		if TF_base_functions.export_settings["game"] != "TPF2":
			return
		
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport save_model_data")
		
		if data["modelObj"]:
			self.__export_anims(data["modelObj"], data["modelData"])
			self.__export_events(data["modelObj"], data["modelData"])
	
	def save_mesh_data_always(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport save_mesh_data")
		
		if data["meshObj"]:
			self.__export_anims(data["meshObj"], data["meshData"], True)

	def save_group_data(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport save_group_data")
		
		if data["groupObj"]:
			self.__export_anims(data["groupObj"], data["groupData"])
			self.__export_events(data["groupObj"], data["groupData"])

	def save_lod_data(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport save_lod_data")
		
		if data["lodObj"]:
			self.__export_anims(data["lodObj"], data["lodData"])
			self.__export_events(data["lodObj"], data["lodData"])

	# def load_mesh_data(self, data):
		# if TF_base_functions.import_settings["game"] != "TPF2":
			# return
		
		# if TF_base_functions.user_prefs["debug"]:
			# print("TFAnimImportExport load_mesh_data")
		
		# if data["meshData"]:
			# self.__import_anims(data["meshObj"], data["meshData"], data)
	
	def load_model_2(self, data):
		if TF_base_functions.import_settings["game"] != "TPF2":
			return
		
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport load_model_2")
		
		if data["modelData"]:
			self.__import_anims(data["modelObj"], data["modelData"], data)
	
	def load_mesh_after_creation(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport load_mesh_after_creation")
		
		if data["meshObj"]:
			for i in range(len(data["tris"])):
				if "jointWeights" in data["tris"][i]:
					for j in range(len(data["tris"][i]["jointWeights"])):
						for k in range(3):
							for n in range(4):
								vertIndex = data["tris"][i]["jointWeights"][j][k]
								jointWeight = data["vertexData"]["jointWeights"][vertIndex][n]
								groupIndex = int(jointWeight)
								groupWeight = (jointWeight - groupIndex)/0.999
								if groupWeight > 0.001:
									if groupIndex >= len(data["meshObj"].vertex_groups):
										for m in range(len(data["meshObj"].vertex_groups), groupIndex+1):
											data["meshObj"].vertex_groups.new(str(m))
									
									data["meshObj"].vertex_groups[groupIndex].add([data["faceToVertex"][i][j][k]], groupWeight, 'ADD')
			data["meshData"]["__VertexGroupsCreated"] = True

	def load_mesh_always(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport load_mesh_always")
		
		if data["meshObj"]:
			if "__VertexGroupsCreated" not in data["meshData"] or not data["meshData"]["__VertexGroupsCreated"]:
				# check for vertex groups
				vertex_group_max = 0
				for obj in bpy.data.objects:
					if obj.data == data["meshObj"].data and len(obj.vertex_groups) > vertex_group_max:
						vertex_group_max = len(obj.vertex_groups)

				# add vertex groups
				if vertex_group_max > 1:
					for i in range(vertex_group_max):
						data["meshObj"].vertex_groups.new(str(i))
			
			if TF_base_functions.import_settings["game"] != "TPF2":
				self.__import_anims(data["meshObj"], data["meshData"], data, True)

	def load_group(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport load_group")
		
		if data["groupObj"]:
			self.__import_anims(data["groupObj"], data["groupData"], data)
			self.__import_events(data["groupObj"], data["groupData"])

	def load_lod(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFAnimImportExport load_lod")
		
		if data["lodObj"]:
			self.__import_anims(data["lodObj"], data["lodData"], data)
			self.__import_events(data["lodObj"], data["lodData"])

	def get_mesh_vertex_data(self, data):
		# get vertex weights
		if data["meshObj"] and len(data["meshObj"].vertex_groups) > 1:
			weights = []
			for vertex_group in data["vertex"].groups:
				weights.append(vertex_group.group + vertex_group.weight*0.999)
				if len(weights) == 4:
					break
			for l in range(len(weights), 4):
				weights.append(0)
			data["vectors"]["jointWeights"] = Vector(weights)

	def create_mesh_data_storage(self, data):
		if "jointWeights" not in data["vertexStruct"]:
			data["vertexStruct"]["jointWeights"] = {"count": 0, "offset": 0, "numComp": 4}

class_list = [
	TFAnimSetting,
	TFAnimAddOp,
	TFAnimRemoveOp,
	TFAnimSettingsItemUIList,
	TFEventEntry,
	TFEventEntryAddOp,
	TFEventEntryRemoveOp,
	TFEventEntryItemUIList,
	TFEventSetting,
	TFEventAddOp,
	TFEventRemoveOp,
	TFEventSettingsItemUIList
]

def register():
	for class_type in class_list:
		bpy.utils.register_class(class_type)

	importexport = TFAnimImportExport()
	TF_base_functions.plugin_add("TFAnimImportExport", importexport)


def unregister():
	TF_base_functions.plugin_remove("TFAnimImportExport")

	for i in range(len(class_list)-1, -1, -1):
		bpy.utils.unregister_class(class_list[i])