import bpy
from ..lua import writer
from .base_functions import TF_base_functions, TFPropertyGroup

# Texturelist for different material types
_material_types = [
	["REFLECTIVE_NRML_MAP", "REFLECTIVE_NRML_MAP", "", 0, ["map_color_reflect", "map_normal"], ["map_env"]],
	["REFLECTIVE", "REFLECTIVE", "", 1, ["map_color_reflect"], ["map_env"]],
	["TRANSPARENT_NRML_MAP_OP", "TRANSPARENT_NRML_MAP_OP", "", 2, ["map_color_alpha", "map_normal", "map_op"]],
	["TRANSPARENT_NRML_MAP", "TRANSPARENT_NRML_MAP", "", 3, ["map_color_alpha", "map_normal"]],
	["TRANSPARENT_OP", "TRANSPARENT_OP", "", 4, ["map_color_alpha", "map_op"]],
	["TRANSPARENT", "TRANSPARENT", "", 5, ["map_color_alpha"]],
	["REFLECTIVE_OP", "REFLECTIVE_OP", "", 6, ["map_color_reflect", "map_op"], ["map_env"]],
	["REFLECTIVE_NRML_MAP_OP", "REFLECTIVE_NRML_MAP_OP", "", 7, ["map_color_reflect", "map_normal", "map_op"], ["map_env"]],
	["TRANSPARENT_DETAIL", "TRANSPARENT_DETAIL", "", 8, ["map_color_alpha", "map_detail"]],
	["LEAF_CARD", "LEAF_CARD", "", 9, ["map_color_alpha", "map_normal"]],
	["BILLBOARD", "BILLBOARD", "", 10, ["map_color_alpha", "map_normal"]],
	["SKINNING_TEXTURED", "SKINNING_TEXTURED", "", 11, ["map_color"]],
	["PHYSICAL", "PHYSICAL", "", 12, ["map_albedo", "map_metal_gloss_ao"]],
	["PHYSICAL_NRML_MAP", "PHYSICAL_NRML_MAP", "", 13, ["map_albedo", "map_normal", "map_metal_gloss_ao"]],
	["PHYSICAL_NRML_MAP_CBLEND", "PHYSICAL_NRML_MAP_CBLEND", "", 14, ["map_albedo", "map_normal", "map_metal_gloss_ao", "map_cblend"]],
	["PHYSICAL_NRML_MAP_CBLEND_DIRT", "PHYSICAL_NRML_MAP_CBLEND_DIRT", "", 15, ["map_albedo", "map_normal", "map_metal_gloss_ao", "map_cblend_dirt_rust"], ["map_dirt", "map_dirt_normal", "map_rust", "map_rust_normal"]],
	["PHYSICAL_NRML_MAP_OP", "PHYSICAL_NRML_MAP_OP", "", 16, ["map_albedo", "map_normal", "map_metal_gloss_ao", "map_op"]],
	["PHYSICAL_NRML_MAP_CBLEND_OP", "PHYSICAL_NRML_MAP_CBLEND_OP", "", 17, ["map_albedo", "map_normal", "map_metal_gloss_ao", "map_cblend", "map_op"]],
	["PHYSICAL_NRML_MAP_UV1_AO", "PHYSICAL_NRML_MAP_UV1_AO", "", 18, ["map_albedo", "map_normal", "map_metal_gloss_ao"]],
	["PHYSICAL_GLOSS_ONLY", "PHYSICAL_GLOSS_ONLY", "", 19, ["map_albedo_gloss"]],
	["PHYSICAL_ALPHA_NRML_MAP", "PHYSICAL_ALPHA_NRML_MAP", "", 20, ["map_albedo_alpha", "map_normal"]],
	["PHYSICAL_ALPHA_NRML_MAP_OP", "PHYSICAL_ALPHA_NRML_MAP_OP", "", 21, ["map_albedo_alpha", "map_normal", "map_op"]],
	["PHYS_TRANSPARENT", "PHYS_TRANSPARENT", "", 22, ["map_albedo_opacity", "map_metal_gloss_ao"]],
	["PHYS_TRANSPARENT_NRML_MAP", "PHYS_TRANSPARENT_NRML_MAP", "", 23, ["map_albedo_opacity", "map_normal", "map_metal_gloss_ao"]],
	["PHYS_TRANSPARENT_NRML_MAP_CBLEND_DIRT", "PHYS_TRANSPARENT_NRML_MAP_CBLEND_DIRT", "", 24, ["map_albedo_opacity", "map_normal", "map_metal_gloss_ao", "map_cblend_dirt_rust"], ["map_dirt", "map_dirt_normal", "map_rust", "map_rust_normal"]],
	["PHYSICAL_NRML_MAP_CBLEND_DIRT_LOGO", "PHYSICAL_NRML_MAP_CBLEND_DIRT_LOGO", "", 25, ["map_albedo", "map_normal", "map_metal_gloss_ao", "map_cblend_dirt_rust"], ["map_dirt", "map_dirt_normal", "map_rust", "map_rust_normal", "map_logo"]],
	["EMISSIVE", "EMISSIVE", "", 26, ["map_emissive"]],
	["NONE", "NONE", "", 27, []]
]

_default_maps = {
	"map_dirt": {
		"compressionAllowed": True,
		"image": "dirt_albedo.dds",
		"model_type": "models/vehicle/",
		"magFilter": "LINEAR",
		"minFilter": "LINEAR_MIPMAP_LINEAR",
		"mipmapAlphaScale": 0,
		"type": "TWOD",
		"wrapS": "REPEAT",
		"wrapT": "REPEAT",
	},
	"map_dirt_normal": {
		"compressionAllowed": True,
		"image": "dirt_normal.dds",
		"model_type": "models/vehicle/",
		"magFilter": "LINEAR",
		"minFilter": "LINEAR_MIPMAP_LINEAR",
		"mipmapAlphaScale": 0,
		"type": "TWOD",
		"wrapS": "REPEAT",
		"wrapT": "REPEAT",
	},
	"map_rust": {
		"compressionAllowed": True,
		"image": "rust_albedo.dds",
		"model_type": "models/vehicle/",
		"magFilter": "LINEAR",
		"minFilter": "LINEAR_MIPMAP_LINEAR",
		"mipmapAlphaScale": 0,
		"type": "TWOD",
		"wrapS": "REPEAT",
		"wrapT": "REPEAT",
	},
	"map_rust_normal": {
		"compressionAllowed": True,
		"image": "rust_normal.dds",
		"model_type": "models/vehicle/",
		"magFilter": "LINEAR",
		"minFilter": "LINEAR_MIPMAP_LINEAR",
		"mipmapAlphaScale": 0,
		"type": "TWOD",
		"wrapS": "REPEAT",
		"wrapT": "REPEAT",
	},
	"map_env": {
		"compressionAllowed": True,
		"image": "c.tga",
		"model_type": "",
		"magFilter": "LINEAR",
		"minFilter": "LINEAR",
		"mipmapAlphaScale": 0,
		"type": "CUBE_MAP",
		"wrapS": "REPEAT",
		"wrapT": "REPEAT",
	},
}

_material_type_list = []
for mat_type in _material_types:
	_material_type_list.append(mat_type[0])

# Import
def _create_material_blender(material, data, basepath, errors):
	if data["material"]["type"] in texturetypes:
		for tex_type in texturetypes[data["material"]["type"]]:
			if tex_type in data["material"]["params"]:
				cur_tex = data["material"]["params"][tex_type]
				texture = bpy.data.textures.new(tex_type, 'IMAGE')
				texture.use_alpha = False
				texture.tf_texture.texture_type = tex_type
				if cur_tex["wrapS"] == "REPEAT":
					texture.extension = "REPEAT"
				elif cur_tex["wrapS"] == "CLAMP_TO_EDGE":
					texture.extension = "EXTEND"
				else:
					texture.extension = "REPEAT"
				tex_filename = cur_tex["fileName"]
				texture.model_type = tex_filename.partition("models/")[2].rpartition("/")[0]+"/"
				image = TF_base_functions.get_data(basepath + "textures/" + tex_filename, "IMAGE", errors)

				if image:
					image.use_alpha = False
					texture.image = image
				newtexture = material.texture_slots.add()
				newtexture.texture_coords = "UV"
				if tex_type in {"map_color_reflect", "map_color_alpha", "map_color"}:
					newtexture.use_map_color_diffuse = True
					newmaterial.use_face_texture = True
					newmaterial.use_face_texture_alpha = False

					if tex_type == "map_color_alpha":
						newmaterial.use_face_texture_alpha = True
						texture.use_alpha = True
						if texture.image:
							texture.image.use_alpha = True

				elif tex_type == "map_normal":
					texture.use_normal_map = True

					newtexture.use_map_normal = True
					newtexture.use_map_color_diffuse = False
					newtexture.normal_map_space = "TANGENT"

				newtexture.texture = texture

			#else:
				#self.report({error_levels[error_level][2]}, "Required texture \"%s\" for materialtype \"%s\" missing in material \"%s\"!" %(tex_type, data["material"]["type"], data["material"]["name"]))
	#else:
		#self.report({error_levels[error_level][2]}, "Unknown materialtype \"%s\" in material \"%s\"!" %(data["material"]["type"], data["material"]["name"]))

"""
def _create_material_nodes():
	newmaterial.use_nodes = True
			nodeTree = newmaterial.node_tree

			blender_nodes = {}
			cycles_nodes = {}
			if self.renderer == 'CYCLES':
				cycles_nodes['material_out'] = nodeTree.nodes[0]
				cycles_nodes['diffuse'] = nodeTree.nodes[1]
				if not 'CYCLES' in self.renderers:
					nodeTree.nodes.remove(cycles_nodes['material_out'])
					nodeTree.nodes.remove(cycles_nodes['diffuse'])
					cycles_nodes = {}
			else:
				blender_nodes['output'] = nodeTree.nodes[0]
				blender_nodes['material'] = nodeTree.nodes[1]
				if not 'NODES' in self.renderers:
					nodeTree.nodes.remove(blender_nodes['output'])
					nodeTree.nodes.remove(blender_nodes['material'])
					blender_nodes = {}


			if 'CYCLES' in self.renderers:
				if not 'material_out' in cycles_nodes:
					cycles_nodes['material_out'] = nodeTree.nodes.new(type="ShaderNodeOutputMaterial")
				if not 'diffuse' in cycles_nodes:
					cycles_nodes['diffuse'] = nodeTree.nodes.new(type="ShaderNodeBsdfDiffuse")

				nodeTree.links.new(cycles_nodes['material_out'].inputs[0], cycles_nodes['diffuse'].outputs[0])

				cycles_nodes['frame'] = nodeTree.nodes.new(type="NodeFrame")
				cycles_nodes['frame'].label = "Cycles"
				nodeTree.nodes.active = cycles_nodes['frame']

				cycles_nodes['material_out'].parent = cycles_nodes['frame']
				cycles_nodes['material_out'].location = Vector((250, 300))
				cycles_nodes['material_out'].select = False

				cycles_nodes['diffuse'].parent = cycles_nodes['frame']
				cycles_nodes['diffuse'].location = Vector((50, 300))
				cycles_nodes['diffuse'].select = False

			if 'NODES' in self.renderers:
				if not 'output' in blender_nodes:
					blender_nodes['output'] = nodeTree.nodes.new(type="ShaderNodeOutput")
				if not 'material' in blender_nodes:
					blender_nodes['material'] = nodeTree.nodes.new(type="ShaderNodeMaterial")

				nodeTree.links.new(blender_nodes['output'].inputs[0], blender_nodes['material'].outputs[0])

				blender_nodes['frame'] = nodeTree.nodes.new(type="NodeFrame")
				blender_nodes['frame'].label = "Blender Render"
				nodeTree.nodes.active = blender_nodes['frame']

				blender_nodes['output'].parent = blender_nodes['frame']
				blender_nodes['output'].location = Vector((250, 300))
				blender_nodes['output'].select = False

				blender_nodes['material'].parent = blender_nodes['frame']
				blender_nodes['material'].location = Vector((50, 300))
				blender_nodes['material'].select = False



		for tex_type in texturetypes[material_data[i][j]['type']]:
			if tex_type in material_data[i][j]['params']:
				cur_tex = material_data[i][j]['params'][tex_type]
				texture = bpy.data.textures.new(tex_type, 'IMAGE')
				texture.use_alpha = False
				texture.tf_texture.texture_type = tex_type
				if cur_tex['wrapS'] == "REPEAT":
					texture.extension = 'REPEAT'
				elif cur_tex['wrapS'] == "CLAMP_TO_EDGE":
					texture.extension = 'EXTEND'
				else:
					texture.extension = 'REPEAT'
				tex_filename = cur_tex['fileName']
				texture.model_type = tex_filename.partition('models/')[2].rpartition('/')[0]+'/'
				image = self.get_data(basepath + 'textures/' + tex_filename, "IMAGE", errors)

				if image:
					image.use_alpha = False
					texture.image = image
				if 'BLENDER' in self.renderers:
					newtexture = newmaterial.texture_slots.add()
					newtexture.texture_coords = 'UV'
				if tex_type in {'map_color_reflect', 'map_color_alpha', 'map_color'}:
					if j == matConfig[i]:
						material_diffuse_image = texture.image

					if 'BLENDER' in self.renderers:
						newtexture.use_map_color_diffuse = True
						newmaterial.use_face_texture = True
						newmaterial.use_face_texture_alpha = False

					if 'CYCLES' in self.renderers:
						cycles_nodes['diffuseImg'] = nodeTree.nodes.new(type="ShaderNodeTexImage")
						cycles_nodes['diffuseImg'].color_space = 'NONE'
						cycles_nodes['diffuseImg'].image = texture.image
						cycles_nodes['diffuseImg'].parent = cycles_nodes['frame']
						cycles_nodes['diffuseImg'].location = Vector((-150, 400))
						cycles_nodes['diffuseImg'].select = False
						nodeTree.links.new(cycles_nodes['diffuse'].inputs[0], cycles_nodes['diffuseImg'].outputs[0])

					if tex_type == 'map_color_alpha':
						if 'BLENDER' in self.renderers:
							newmaterial.use_face_texture_alpha = True
						texture.use_alpha = True
						if texture.image:
							texture.image.use_alpha = True

				elif tex_type == 'map_normal':
					texture.use_normal_map = True

					if 'BLENDER' in self.renderers:
						newtexture.use_map_normal = True
						newtexture.use_map_color_diffuse = False
						newtexture.normal_map_space = 'TANGENT'

					if 'CYCLES' in self.renderers:
						cycles_nodes['normal'] = nodeTree.nodes.new(type="ShaderNodeNormalMap")
						cycles_nodes['normal'].uv_map = uvmap.name
						cycles_nodes['normal'].parent = cycles_nodes['frame']
						cycles_nodes['normal'].location = Vector((-150, 150))
						cycles_nodes['normal'].select = False
						nodeTree.links.new(cycles_nodes['diffuse'].inputs[2], cycles_nodes['normal'].outputs[0])

						cycles_nodes['normalImg'] = nodeTree.nodes.new(type="ShaderNodeTexImage")
						cycles_nodes['normalImg'].color_space = 'NONE'
						cycles_nodes['normalImg'].image = texture.image
						cycles_nodes['normalImg'].parent = cycles_nodes['frame']
						cycles_nodes['normalImg'].location = Vector((-350, 150))
						cycles_nodes['normalImg'].select = False
						nodeTree.links.new(cycles_nodes['normal'].inputs[1], cycles_nodes['normalImg'].outputs[0])

				if 'BLENDER' in self.renderers:
					newtexture.texture = texture

			else:
				self.report({error_levels[error_level][2]}, "Required texture \"%s\" for materialtype \"%s\" missing in material \"%s\"!" %(tex_type, material_data[i][j]['type'], material_data[i][j]['name']))
"""

#def _create_material_cycles():
def _validate_data(data, errors):
	TF_base_functions.call_plugin("validate_material_data", data, errors)
	if data["materialData"]["type"] not in _material_type_list:
		errors.append({"level": 2, "message": "Unknown material type \"%s\" in Material \"%s\"!" % (data["materialData"]["type"], data["fileName"])})
	else:
		for param in data["materialData"]["params"].keys():
			if param[0:4] == "map_" and param != "map_logo" and "fileName" not in data["materialData"]["params"][param]:
				errors.append({"level": 2, "message": "Missing filename for texture \"%s\" in Material \"%s\"!" % (param, data["fileName"])})
	return True

def load_from_file(mat_filename, errors):
	basepath, filepath, name = TF_base_functions.split_path(mat_filename)
	if basepath == "":
		errors.append({"level": 1, "message": "Could not find \"res\"-directory in \"%s\", loading of textures probably won't be succesfull." %mat_filename})
		basepath = TF_base_functions.import_settings["basepath"] + "/res/"
	
	name = name.replace(".mtl", "")
	model_type = TF_base_functions.get_model_type(mat_filename, "models/material/")

	material_data = TF_base_functions.get_data(mat_filename, "MATERIAL", errors)

	data = {
		"materialData": material_data,
		"fileName": mat_filename,
		"name": name,
		"basepath": basepath,
	}
	TF_base_functions.call_plugin("load_material_data", data, errors)

	k = bpy.data.materials.find(name)
	if k >= 0 and bpy.data.materials[k].model_type == model_type:
		material = bpy.data.materials[k]
	else:
		material = bpy.data.materials.new(name)
		material.model_type = model_type

		if _validate_data(data, errors):
			material.tf_material.material_type = material_data["type"]
			for param in material_data["params"].keys():
				if param[0:4] == "map_":
					# textures
					if param == "map_logo":
						# to-do: logos
						pass
					else:
						texture_data = material_data["params"][param]
						texture = material.tf_material._get_texture(param)
						texture.model_type = TF_base_functions.get_model_type(basepath + "textures/" + texture_data["fileName"], "textures/")

						for key in texture_data:
							if key == "fileName":
								if "type" in texture_data and texture_data["type"] == "CUBE_MAP":
									image = None
								else:
									image = TF_base_functions.get_data(basepath + "textures/" + texture_data["fileName"], "IMAGE", errors)

								if image:
									texture.image = image.name
							else:
								if hasattr(texture, key):
									setattr(texture, key, texture_data[key])

				elif hasattr(material.tf_material, param):
					# parameters
					material_param = getattr(material.tf_material, param)
					for key in material_data["params"][param].keys():
						if hasattr(material_param, key):
							setattr(material_param, key, material_data["params"][param][key])

		material.use_fake_user = True

		data["material"] = material
		TF_base_functions.call_plugin("load_material", data, errors)

	return material


def get_diffuse_texture(material):
	image = None
	material_type = material.tf_material.material_type

	if material_type != "NONE" and material_type in _material_type_list:
		diffuse_texture = _material_types[_material_type_list.index(material_type)][4][0]
		if diffuse_texture in material.tf_material.textures and material.tf_material.textures[diffuse_texture].image in bpy.data.images:
			image = bpy.data.images[material.tf_material.textures[diffuse_texture].image]

	return image

# Export
def create_file_data(material_object, name, errors):
	material = material_object["object"]

	mat_name = name
	if mat_name == "":
		mat_name =  TF_base_functions.export_settings["prefix"] + material.name

	if (TF_base_functions.user_prefs["debug"]):
		print("\tsave_material_file", mat_name)

	if "materials" not in TF_base_functions.export_settings["filedata"]:
		TF_base_functions.export_settings["filedata"]["materials"] = {}
	filedata = TF_base_functions.export_settings["filedata"]["materials"]

	if "images" not in TF_base_functions.export_settings["filedata"]:
		TF_base_functions.export_settings["filedata"]["images"] = {}
	image_filedata = TF_base_functions.export_settings["filedata"]["images"]

	model_type = material.model_type
	if model_type == "%auto":
		model_type = TF_base_functions.export_settings["model_type"]
	elif model_type == "%inherit" and "parent" in material_object and "model_type" in material_object["parent"]:
		model_type = material_object["parent"]["model_type"]

	create_material = TF_base_functions.export_settings["subfiles"] and TF_base_functions.export_settings["material_files"]

	if create_material and model_type+mat_name not in filedata:
		material_data = {}
		material_data["type"] = material.tf_material.material_type

		if material_data["type"] in _material_type_list:
			material_data["params"] = TF_base_functions.prop_to_value(material.tf_material)
			index = _material_type_list.index(material_data["type"])
			textures = _material_types[index][4].copy()
			if len(_material_types[index]) > 5:
				textures += _material_types[index][5]
			
			# delete unused params
			if "map_cblend" not in textures and "map_cblend_dirt_rust" not in textures and "color_blend" in material_data["params"]:
				del material_data["params"]["color_blend"]
			if "map_cblend_dirt_rust" not in textures and "dirt_rust" in material_data["params"]:
				del material_data["params"]["dirt_rust"]
			
			for texture_type in textures:
				if texture_type == "map_logo":
					material_data["params"][texture_type] = "player-logo-array"
				else:
					texture = material.tf_material._get_texture(texture_type)
					if texture:
						image_model_type = texture.model_type
						if image_model_type == "%auto":
							image_model_type = "models/" + TF_base_functions.export_settings["model_type"]
						elif image_model_type == "%inherit":
							image_model_type = "models/" + model_type

						material_data["params"][texture_type] = {
							"fileName": image_model_type +  TF_base_functions.export_settings["prefix"] + texture.image.partition(".")[0] + ".tga",
							"compressionAllowed": texture.compressionAllowed,
							"magFilter": texture.magFilter,
							"minFilter": "LINEAR_MIPMAP_LINEAR",
							"mipmapAlphaScale": texture.mipmapAlphaScale,
							"type": texture.type,
							"wrapS": texture.wrapS,
							"wrapT": texture.wrapT,
						}
						if texture.image in bpy.data.images:
							if material_data["params"][texture_type]["fileName"] not in image_filedata:
								filepath = "textures/" + material_data["params"][texture_type]["fileName"]
								image_filedata[material_data["params"][texture_type]["fileName"]] = {
									"image": bpy.data.images[texture.image],
									"path": filepath,
									"textureType": texture_type,
									"matType": material_data["type"],
								}
						elif texture_type not in _default_maps:
							errors.append({"level": 2, "message": "Map \"%s\" in material \"%s\" has no texture!" % (texture_type, mat_name)})
					else:
						errors.append({"level": 2, "message": "Required map \"%s\" not found in material \"%s\"!" % (texture_type, mat_name)})

			del textures

		else:
			# unknown mat type
			errors.append({"level": 2, "message": "Unknown type \"%s\" for material \"%s\"!" % (material_data["type"], mat_name)})

		data = {
			"materialObject": material,
			"materialData": material_data,
		}
		TF_base_functions.call_plugin("save_material_data", data, errors)
		filedata[model_type+mat_name] = {"data": material_data, "model_type": model_type, "name": mat_name}

	return model_type+mat_name+".mtl"

def write_file(file, errors):
	filename = file["name"] + ".mtl"
	filepath = file["model_type"]

	id = filepath + filename

	resfilepath = "models/material/" + filepath
	resfilename = resfilepath + filename

	if TF_base_functions.export_settings["dir_structure"]:
		osfilepath = TF_base_functions.export_settings["basepath"]+resfilepath
	else:
		osfilepath = TF_base_functions.export_settings["basepath"]

	osfilename = osfilepath + filename

	if (not writer.isfile(osfilename)) or TF_base_functions.export_settings["override"] or ("forceOverride" in file and file["forceOverride"]):
		if not writer.exists(osfilepath):
			writer.makedirs(osfilepath)

		filetext = ""
		# additional code, to do: not restricted to require
		if "__requires" in file["data"]:
			for req in file["data"]["__requires"]:
				filetext += req + '\n'

		filetext += "function data()\nreturn "
		filetext += writer.write_var_struct_string(file["data"], "", 0, True, False)
		filetext += "end"

		data = {
			"text": filetext,
			"materialData": file["data"],
			"fileID": id,
		}
		TF_base_functions.call_plugin("write_material_file", data, errors)

		output = open(osfilename, 'w', encoding="utf-8")
		output.write(data["text"])
		output.close()

	TF_base_functions.export_settings["filelist"].append([osfilename, "res/" + resfilename])

# UI functions
# Classes

class TFTextureSetting(bpy.types.PropertyGroup):
	def _getWrapS(self):
		if "wrapS" in self:
			if self["wrapS"] == "REPEAT":
				return 1
		return 0

	def _getWrapT(self):
		if "wrapT" in self:
			if self["wrapT"] == "REPEAT":
				return 1
		return 0

	def _setWrapS(self, value):
		if value == 1:
			self["wrapS"] = "REPEAT"
		else:
			self["wrapS"] = "CLAMP_TO_EDGE"

	def _setWrapT(self, value):
		if value == 1:
			self["wrapT"] = "REPEAT"
		else:
			self["wrapT"] = "CLAMP_TO_EDGE"

	name = bpy.props.StringProperty(default="")
	image = bpy.props.StringProperty(default="")
	model_type = bpy.props.StringProperty(default="%auto")
	compressionAllowed = bpy.props.BoolProperty(default=False)
	magFilter = bpy.props.EnumProperty(items = [
			("LINEAR", "LINEAR", "", 0),
			("NEAREST", "NEAREST", "", 1),
		],)
	mipmapAlphaScale = bpy.props.FloatProperty()
	type = bpy.props.EnumProperty(items = [
			("TWOD", "TWOD", "", 0),
			("CUBE_MAP", "CUBE_MAP", "", 1),
			("TWOD_ARRAY", "TWOD_ARRAY", "", 2),
		],)
	wrapS = bpy.props.EnumProperty(
		items = [
			("CLAMP_TO_EDGE", "CLAMP_TO_EDGE", "", 0),
			("REPEAT", "REPEAT", "", 1),
		],
		set = _setWrapS,
		get = _getWrapS
	)
	wrapT = bpy.props.EnumProperty(
		items = [
			("CLAMP_TO_EDGE", "CLAMP_TO_EDGE", "", 0),
			("REPEAT", "REPEAT", "", 1),
		],
		set = _setWrapT,
		get = _getWrapT
	)

	@classmethod
	def register(cls):
		TFMaterialSetting.textures = bpy.props.CollectionProperty(type = cls)
		TFMaterialSetting.textures_activeID = bpy.props.IntProperty()

	@classmethod
	def unregister(cls):
		del TFMaterialSetting.textures
		del TFMaterialSetting.textures_activeID

class TFTextureSettingsPanel(bpy.types.Panel):
	bl_label = "Train Fever Texture"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "material"

	@classmethod
	def poll(cls, context):
		if (context and context.object and context.object.type in {'MESH'}):
			if context.object.active_material:
				return True
		return False

	def draw(self, context):
		self.draw_texture(context, self.layout)

	def draw_texture(self, context, layout):
		mydata = context.object.active_material

		if mydata:
			# texture
			boxTexture = TF_base_functions.draw_hideable_box(layout, "Texture:", "material_texture", False)
			if boxTexture:
				if mydata.tf_material.textures_activeID < len(mydata.tf_material.textures):
					texture = mydata.tf_material.textures[mydata.tf_material.textures_activeID]
					boxTexture.label(str(texture.name))
					boxTexture.prop_search(texture, "image", bpy.data, "images", text="Image")
					boxTexture.prop(texture, "compressionAllowed", text="Compression allowed")
					boxTexture.prop(texture, "magFilter", text="Mag Filter")
					boxTexture.prop(texture, "mipmapAlphaScale", text="MipMap alpha scale")
					boxTexture.prop(texture, "type", text="Type")
					boxTexture.prop(texture, "wrapS", text="WrapS")
					boxTexture.prop(texture, "wrapT", text="WrapT")

				layout.separator()

			TF_base_functions.call_plugin("draw_texture_settings_panel", {"layout": layout, "mydata": mydata}, [])
			if mydata.tf_material.textures_activeID < len(mydata.tf_material.textures):
				TF_base_functions.draw_internal_box(mydata.tf_material.textures[mydata.tf_material.textures_activeID], layout, "material_texture")


class TFTextureSettingsItemUIList(bpy.types.UIList):
	show_advanced_textures = bpy.props.BoolProperty()
	def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
		layout.label(item.name)

	def draw_filter(self, context, layout):
		layout.prop(self, "show_advanced_textures", text="Advanced textures")

	def filter_items(self, context, data, propname):
		flt_flags = []
		flt_neworder = []

		index = _get_materialType_by_identifier(data.material_type)
		material_textures = _material_types[index][4].copy()
		if len(_material_types[index]) > 5 and self.show_advanced_textures:
			material_textures += _material_types[index][5]
		for texture in getattr(data, propname):
			if texture.name in material_textures:
				flt_flags.append(self.bitflag_filter_item)
				flt_neworder.append(material_textures.index(texture.name))
			else:
				flt_flags.append(0)
				flt_neworder.append(0)

		del material_textures

		return flt_flags, flt_neworder

def _get_materialType_enum_callback(self, context):
	type_list = []
	for mat_type in _material_types:
		type_list.append((mat_type[0], mat_type[1], mat_type[2], mat_type[3]))
	return type_list

def _get_materialType_by_identifier(value):
	for i in range(len(_material_types)):
		if _material_types[i][0] == value:
			return i
	return -1

class TFMaterialEmissiveScale(TFPropertyGroup):
	prop_names = ["emissiveScale"]
	emissiveScale = bpy.props.FloatVectorProperty(default = (1, 1, 1), subtype = 'COLOR')

class TFMaterialCBlend(TFPropertyGroup):
	prop_names = ["albedoScale"]
	albedoScale = bpy.props.FloatProperty(default = 1.0)

class TFMaterialDirtRust(TFPropertyGroup):
	prop_names = [
		"dirtColor",
		"dirtOpacity",
		"dirtScale",
		"rustColor",
		"rustOpacity",
		"rustScale",
	]
	dirtColor = bpy.props.FloatVectorProperty(default = (49/255, 47/255, 35/255), subtype = 'COLOR')
	dirtOpacity = bpy.props.FloatProperty(default = 1.0)
	dirtScale = bpy.props.FloatProperty(default = 1.0)
	rustColor = bpy.props.FloatVectorProperty(default = (74/255, 60/255, 44/255), subtype = 'COLOR')
	rustOpacity = bpy.props.FloatProperty(default = 1.0)
	rustScale = bpy.props.FloatProperty(default = 1.0)

class TFMaterialFadeOutRange(TFPropertyGroup):
	prop_names = ["fadeOutEndDist", "fadeOutStartDist"]
	fadeOutEndDist = bpy.props.IntProperty(default = 20000)
	fadeOutStartDist = bpy.props.IntProperty(default = 10000)

class TFMaterialPolygonOffset(TFPropertyGroup):
	prop_names = ["factor", "forceDepthWrite", "units"]
	factor = bpy.props.IntProperty(default = 0)
	forceDepthWrite = bpy.props.BoolProperty(default = False)
	units = bpy.props.IntProperty(default = 0)

class TFMaterialTwoSided(TFPropertyGroup):
	prop_names = ["flipNormal", "twoSided"]
	flipNormal = bpy.props.BoolProperty(default = False)
	twoSided = bpy.props.BoolProperty(default = False)

class TFMaterialSetting(TFPropertyGroup):
	prop_names = [
		"emissive_scale",
		"color_blend",
		"dirt_rust",
		"fade_out_range",
		"polygon_offset",
		"two_sided",
	]
	def _get_texture(self, name, create = True):
		texture = None
		if name in self.textures:
			texture = self.textures[name]
		else:
			texture = self.textures.add()
			texture.name = name
			
			# get default values
			if name in _default_maps:
				map = _default_maps[name]
				for key in map.keys():
					if hasattr(texture, key):
						setattr(texture, key, map[key])

			#search for old texture definitions
			if self.id_data:
				for tex in self.id_data.texture_slots:
					if tex and name in tex.name:
						# found old texture, copying data
						if tex.texture.image:
							texture.image = tex.texture.image.name

						if "model_type" in tex.texture:
							texture.model_type = tex.texture["model_type"]

						if tex.texture.extension == "REPEAT":
							texture.wrapS = "REPEAT"
							texture.wrapT = "REPEAT"

						if tex.texture.type == "ENVIRONMENT_MAP":
							texture.type = "CUBE_MAP"
						elif tex.texture.image and tex.texture.image.source == "SEQUENCE":
							texture.type = "TWOD_ARRAY"
						break

		return texture

	def _get_material_type(self):
		result = 0
		if "material_type" in self:
			i = _get_materialType_by_identifier(self["material_type"])
			if i >= 0:
				result = i

		material_textures = _material_types[result][4].copy()
		if len(_material_types[result]) > 5:
			material_textures += _material_types[result][5]

		for tex_type in material_textures:
			self._get_texture(tex_type)

		del material_textures

		return result

	def _set_material_type(self, value):
		self["material_type"] = _material_types[value][0]

	material_type = bpy.props.EnumProperty(items=_get_materialType_enum_callback, set=_set_material_type, get=_get_material_type)

	# params
	emissive_scale = bpy.props.PointerProperty(type=TFMaterialEmissiveScale)
	color_blend = bpy.props.PointerProperty(type=TFMaterialCBlend)
	dirt_rust = bpy.props.PointerProperty(type=TFMaterialDirtRust)
	fade_out_range = bpy.props.PointerProperty(type=TFMaterialFadeOutRange)
	polygon_offset = bpy.props.PointerProperty(type=TFMaterialPolygonOffset)
	two_sided = bpy.props.PointerProperty(type=TFMaterialTwoSided)

	@classmethod
	def register(cls):
		bpy.types.Material.tf_material = bpy.props.PointerProperty(type=cls)

	@classmethod
	def unregister(cls):
		del bpy.types.Material.tf_material

class TFMaterialSettingsPanel(bpy.types.Panel):
	bl_label = "Train Fever Material"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "material"

	@classmethod
	def poll(cls, context):
		if (context and context.object and context.object.type in {'MESH'}):
			if context.object.active_material:
				return True
		return False

	def draw(self, context):
		if context.object.type in {'MESH'}:
			self.draw_material(context, self.layout)

	def draw_material(self, context, layout):
		mydata = context.object.active_material

		if mydata:
			layout.prop(mydata.tf_material, "material_type", text="Type")

			material_type = _material_types[_get_materialType_by_identifier(mydata.tf_material.material_type)]
			material_textures = material_type[4].copy()

			if len(material_type) > 5:
				material_textures += material_type[5]

			boxTextures = TF_base_functions.draw_hideable_box(layout, "Textures:", "material_textures", False)
			if boxTextures:
				boxTextures.template_list(
					listtype_name="TFTextureSettingsItemUIList",
					dataptr=mydata.tf_material,
					propname="textures",
					active_dataptr=mydata.tf_material,
					active_propname="textures_activeID",
					rows = 1,
					maxrows = 3
				)
				layout.separator()

			if "emissive_scale" in material_textures:
				boxEmissiveScale = TF_base_functions.draw_hideable_box(layout, "Emissive Scale:", "emissive_scale")
				if boxEmissiveScale:
					boxEmissiveScale.prop(mydata.tf_material.emissive_scale, "emissiveScale", text="Emissive scale")
					layout.separator()
			
			if "map_cblend" in material_textures or "map_cblend_dirt_rust" in material_textures:
				# color blend
				boxCBlend = TF_base_functions.draw_hideable_box(layout, "Color Blend:", "material_cblend", False)
				if boxCBlend:
					boxCBlend.prop(mydata.tf_material.color_blend, "albedoScale", text="Albedo Scale")
					layout.separator()
			if "map_cblend_dirt_rust" in material_textures:
				# dirt
				boxDirt = TF_base_functions.draw_hideable_box(layout, "Dirt:", "material_dirt")
				if boxDirt:
					boxDirt.prop(mydata.tf_material.dirt_rust, "dirtColor", text="Color")
					boxDirt.prop(mydata.tf_material.dirt_rust, "dirtOpacity", text="Opacity")
					boxDirt.prop(mydata.tf_material.dirt_rust, "dirtScale", text="Scale")
					layout.separator()
				# rust
				boxRust = TF_base_functions.draw_hideable_box(layout, "Rust:", "material_rust")
				if boxRust:
					boxRust.prop(mydata.tf_material.dirt_rust, "rustColor", text="Color")
					boxRust.prop(mydata.tf_material.dirt_rust, "rustOpacity", text="Opacity")
					boxRust.prop(mydata.tf_material.dirt_rust, "rustScale", text="Scale")
					layout.separator()

			# advanced parameters
			# fade out range
			boxFadeOut = TF_base_functions.draw_hideable_box(layout, "Fade out Range:", "material_fade_out_range")
			if boxFadeOut:
				boxFadeOut.prop(mydata.tf_material.fade_out_range, "fadeOutStartDist", text="Start Distance")
				boxFadeOut.prop(mydata.tf_material.fade_out_range, "fadeOutEndDist", text="End Distance")
				layout.separator()

			# polygon offset
			boxPolygonOffset = TF_base_functions.draw_hideable_box(layout, "Polygon Offset:", "material_polygon_offset")
			if boxPolygonOffset:
				boxPolygonOffset.prop(mydata.tf_material.polygon_offset, "factor", text="Factor")
				boxPolygonOffset.prop(mydata.tf_material.polygon_offset, "forceDepthWrite", text="Force depth write")
				boxPolygonOffset.prop(mydata.tf_material.polygon_offset, "units", text="Units")
				layout.separator()

			# two sided
			boxTwoSided = TF_base_functions.draw_hideable_box(layout, "Two sided:", "material_two_sided")
			if boxTwoSided:
				boxTwoSided.prop(mydata.tf_material.two_sided, "flipNormal", text="Flip normal")
				boxTwoSided.prop(mydata.tf_material.two_sided, "twoSided", text="Two sided")
				layout.separator()

			TF_base_functions.call_plugin("draw_material_settings_panel", {"layout": layout, "mydata": mydata}, [])
			TF_base_functions.draw_internal_box(mydata, layout, "material")

			del material_textures

__class_list = [
	TFTextureSettingsItemUIList,
	TFMaterialEmissiveScale,
	TFMaterialCBlend,
	TFMaterialDirtRust,
	TFMaterialFadeOutRange,
	TFMaterialPolygonOffset,
	TFMaterialTwoSided,
	TFMaterialSetting,
	TFMaterialSettingsPanel,
	TFTextureSetting,
	TFTextureSettingsPanel,
]

def register():
	# general props: model_type and mesh_dir
	bpy.types.Material.model_type = bpy.props.StringProperty(default="%auto")

	#print("tf_material_io.py register")
	for class_type in __class_list:
		bpy.utils.register_class(class_type)

def unregister():
	#print("tf_material_io.py unregister")
	for i in range(len(__class_list)-1, -1, -1):
		bpy.utils.unregister_class(__class_list[i])

	# general props: model_type and mesh_dir
	del bpy.types.Material.model_type